package demoapp;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@WebServlet("/")
public class MemoryUsageServlet extends HttpServlet {

  private Properties properties;
  private MemoryUsageMonitor memoryUsageMonitor;

  @Override
  protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
    resp.getWriter().write(String.format(
        "[%s] average memory usage is %d",
        getTimestamp(),
        memoryUsageMonitor.getAverage()));
  }

  private String getTimestamp() {
    String pattern = properties.getProperty("time.format");
    return new SimpleDateFormat(pattern).format(new Date());
  }

  @Override
  public void init() throws ServletException {
    try {
      properties = loadProperties();
      memoryUsageMonitor = new MemoryUsageMonitor(
          ApplicationConfiguration.getMaxSamples(),
          ApplicationConfiguration.getSampleDelayMillis());
    }
    catch (IOException e) {
      throw new ServletException(e);
    }
  }

  @Override
  public void destroy() {
    if (memoryUsageMonitor != null) {
      memoryUsageMonitor.close();
    }
  }

  private Properties loadProperties() throws IOException {
    try (InputStream stream = getServletContext().getResourceAsStream("/WEB-INF/MemoryUsageServlet.properties")) {
      Properties properties = new Properties();
      properties.load(stream);
      return properties;
    }
  }
}
